/*
 * Java port of Bullet (c) 2008 Martin Dvorak <jezek2@advel.cz>
 *
 * Bullet Continuous Collision Detection and Physics Library
 * Copyright (c) 2003-2008 Erwin Coumans  http://continuousphysics.com/Bullet/
 *
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from
 * the use of this software.
 * 
 * Permission is granted to anyone to use this software for any purpose, 
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 * 
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software. If you use this software
 *    in a product, an acknowledgment in the product documentation would be
 *    appreciated but is not required.
 * 2. Altered source versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 * 3. This notice may not be removed or altered from any source distribution.
 */

package com.bulletphysics;

import com.bulletphysics.util.ArrayPool;
import com.bulletphysics.linearmath.CProfileManager;
import com.bulletphysics.linearmath.Clock;
import javax.vecmath.Vector3f;

/**
 * Bullet's global variables and constants.
 * 
 * @author jezek2
 */
public class BulletGlobals {
	
	public static final boolean DEBUG = true;
	public static boolean ENABLE_PROFILE = true;
	
	public static final float CONVEX_DISTANCE_MARGIN = 0.04f;
	public static final float FLT_EPSILON = 1.19209290e-07f;
	public static final float SIMD_EPSILON = FLT_EPSILON;
	
	public static final float SIMD_2_PI = 6.283185307179586232f;
	public static final float SIMD_PI = SIMD_2_PI * 0.5f;
	public static final float SIMD_HALF_PI = SIMD_2_PI * 0.25f;
	public static final float SIMD_RADS_PER_DEG = SIMD_2_PI / 360f;
	public static final float SIMD_DEGS_PER_RAD = 360f / SIMD_2_PI;
	public static final float SIMD_INFINITY = Float.MAX_VALUE;

	public static ContactDestroyedCallback gContactDestroyedCallback;
	public static ContactAddedCallback gContactAddedCallback;
	public static ContactProcessedCallback gContactProcessedCallback;
	public static float gContactBreakingThreshold = 0.02f;

	// RigidBody
	public static float gDeactivationTime = 2f;
	public static boolean gDisableDeactivation = false;
	
	public static int gTotalContactPoints;
	
	// GjkPairDetector
	// temp globals, to improve GJK/EPA/penetration calculations
	public static int gNumDeepPenetrationChecks = 0;
	public static int gNumGjkChecks = 0;
	public static int gNumSplitImpulseRecoveries = 0;
	
	public static int gNumAlignedAllocs;
	public static int gNumAlignedFree;
	public static int gTotalBytesAlignedAllocs;	
	
	public static int gPickingConstraintId = 0;
	public static final Vector3f gOldPickingPos = new Vector3f();
	public static float gOldPickingDist = 0.f;
	
	public static int gOverlappingPairs = 0;
	public static int gRemovePairs = 0;
	public static int gAddedPairs = 0;
	public static int gFindPairs = 0;
	
	public static final Vector3f ZERO_VECTOR3 = new Vector3f(0f, 0f, 0f);
	
	public static final Clock gProfileClock = new Clock();

	// JAVA NOTE: added for statistics in applet demo
	public static long stepSimulationTime;
	public static long updateTime;
	
	public static final ArrayPool<float[]> floatArrays = new ArrayPool<float[]>(float.class);
	public static final ArrayPool<int[]> intArrays = new ArrayPool<int[]>(int.class);
	
	public static long profileGetTicks() {
		long ticks = gProfileClock.getTimeMicroseconds();
		return ticks;
	}

	public static float profileGetTickRate() {
		//return 1000000f;
		return 1000f;
	}
	
	public static void pushProfile(String name) {
		if (ENABLE_PROFILE) {
			CProfileManager.startProfile(name);
		}
	}
	
	public static void popProfile() {
		if (ENABLE_PROFILE) {
			CProfileManager.stopProfile();
		}
	}
	
}
